@@ -227,6 +227,9 @@ default_alloc_buffer (GstBufferPool * pool, GstBuffer ** buffer,
   *buffer =
       gst_buffer_new_allocate (priv->allocator, priv->size, &priv->params);
 
+  if (!*buffer)
+    return GST_FLOW_ERROR;
+
   return GST_FLOW_OK;
 }
 
@@ -606,9 +609,9 @@ wrong_config:
  *
  * Set the configuration of the pool. If the pool is already configured, and
  * the configuration haven't change, this function will return %TRUE. If the
- * pool is active, this function will try deactivating it. Buffers allocated
- * form this pool must be returned or else this function will do nothing and
- * return %FALSE.
+ * pool is active, this method will return %FALSE and active configuration
+ * will remain. Buffers allocated form this pool must be returned or else this
+ * function will do nothing and return %FALSE.
  *
  * @config is a #GstStructure that contains the configuration parameters for
  * the pool. A default and mandatory set of parameters can be configured with
@@ -642,18 +645,8 @@ gst_buffer_pool_set_config (GstBufferPool * pool, GstStructure * config)
     goto config_unchanged;
 
   /* can't change the settings when active */
-  if (priv->active) {
-    GST_BUFFER_POOL_UNLOCK (pool);
-    if (!gst_buffer_pool_set_active (pool, FALSE)) {
-      GST_BUFFER_POOL_LOCK (pool);
-      goto was_active;
-    }
-    GST_BUFFER_POOL_LOCK (pool);
-
-    /* not likely but as we released the lock */
-    if (priv->active)
-      goto was_active;
-  }
+  if (priv->active)
+    goto was_active;
 
   /* we can't change when outstanding buffers */
   if (g_atomic_int_get (&priv->outstanding) != 0)
@@ -691,7 +684,7 @@ config_unchanged:
 was_active:
   {
     gst_structure_free (config);
-    GST_WARNING_OBJECT (pool, "can't change config, we are active");
+    GST_INFO_OBJECT (pool, "can't change config, we are active");
     GST_BUFFER_POOL_UNLOCK (pool);
     return FALSE;
   }
@@ -1046,9 +1039,11 @@ gst_buffer_pool_config_get_allocator (GstStructure * config,
  * Validate that changes made to @config are still valid in the context of the
  * expected parameters. This function is a helper that can be used to validate
  * changes made by a pool to a config when gst_buffer_pool_set_config()
- * returns %FALSE. This expects that @caps and @size haven't changed, and that
- * @min_buffers aren't lower then what we initially expected. This does not check
- * if options or allocator parameters.
+ * returns %FALSE. This expects that @caps haven't changed and that
+ * @min_buffers aren't lower then what we initially expected.
+ * This does not check if options or allocator parameters are still valid,
+ * won't check if size have changed, since changing the size is valid to adapt
+ * padding.
  *
  * Since: 1.4
  *
@@ -1066,7 +1061,7 @@ gst_buffer_pool_config_validate_params (GstStructure * config, GstCaps * caps,
 
   gst_buffer_pool_config_get_params (config, &newcaps, &newsize, &newmin, NULL);
 
-  if (gst_caps_is_equal (caps, newcaps) && (size == newsize)
+  if (gst_caps_is_equal (caps, newcaps) && (newsize >= size)
       && (newmin >= min_buffers))
     ret = TRUE;
 
@@ -1225,17 +1220,17 @@ default_release_buffer (GstBufferPool * pool, GstBuffer * buffer)
       GST_MINI_OBJECT_FLAGS (buffer));
 
   /* memory should be untouched */
-  if (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_TAG_MEMORY))
-    goto discard;
+  if (G_UNLIKELY (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_TAG_MEMORY)))
+    goto memory_tagged;
 
   /* size should have been reset. This is not a catch all, pool with
    * size requirement per memory should do their own check. */
-  if (gst_buffer_get_size (buffer) != pool->priv->size)
-    goto discard;
+  if (G_UNLIKELY (gst_buffer_get_size (buffer) != pool->priv->size))
+    goto size_changed;
 
   /* all memory should be exclusive to this buffer (and thus be writable) */
-  if (!gst_buffer_is_all_memory_writable (buffer))
-    goto discard;
+  if (G_UNLIKELY (!gst_buffer_is_all_memory_writable (buffer)))
+    goto not_writable;
 
   /* keep it around in our queue */
   gst_atomic_queue_push (pool->priv->queue, buffer);
@@ -1243,6 +1238,25 @@ default_release_buffer (GstBufferPool * pool, GstBuffer * buffer)
 
   return;
 
+memory_tagged:
+  {
+    GST_CAT_DEBUG_OBJECT (GST_CAT_PERFORMANCE, pool,
+        "discarding buffer %p: memory tag set", buffer);
+    goto discard;
+  }
+size_changed:
+  {
+    GST_CAT_DEBUG_OBJECT (GST_CAT_PERFORMANCE, pool,
+        "discarding buffer %p: size %" G_GSIZE_FORMAT " != %u",
+        buffer, gst_buffer_get_size (buffer), pool->priv->size);
+    goto discard;
+  }
+not_writable:
+  {
+    GST_CAT_DEBUG_OBJECT (GST_CAT_PERFORMANCE, pool,
+        "discarding buffer %p: memory not writable", buffer);
+    goto discard;
+  }
 discard:
   {
     do_free_buffer (pool, buffer);
