@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifndef __GST_BASE_TRANSFORM_H__
@@ -129,16 +129,16 @@ struct _GstBaseTransform {
  *                    downstream allocation query. This function is only called
  *                    when not operating in passthrough mode. The default
  *                    implementation will remove all memory dependent metadata.
- *                    If there is ia @filter_meta method implementation, it will
+ *                    If there is a @filter_meta method implementation, it will
  *                    be called for all metadata API in the downstream query,
  *                    otherwise the metadata API is removed.
- * @filter_meta: Return TRUE if the metadata API should be proposed in the
- *               upstream allocation query. The default implementation is NULL
+ * @filter_meta: Return %TRUE if the metadata API should be proposed in the
+ *               upstream allocation query. The default implementation is %NULL
  *               and will cause all metadata to be removed.
  * @propose_allocation: Propose buffer allocation parameters for upstream elements.
  *                      This function must be implemented if the element reads or
  *                      writes the buffer content. The query that was passed to
- *                      the decide_allocation is passed in this method (or NULL
+ *                      the decide_allocation is passed in this method (or %NULL
  *                      when the element is in passthrough mode). The default
  *                      implementation will pass the query downstream when in
  *                      passthrough mode and will copy all the filtered metadata
@@ -176,9 +176,9 @@ struct _GstBaseTransform {
  *                 The default implementation will copy the flags, timestamps and
  *                 offsets of the buffer.
  * @transform_meta: Optional. Transform the metadata on the input buffer to the
- *                  output buffer. By default this method is NULL and no
+ *                  output buffer. By default this method is %NULL and no
  *                  metadata is copied. subclasses can implement this method and
- *                  return TRUE if the metadata is to be copied.
+ *                  return %TRUE if the metadata is to be copied.
  * @before_transform: Optional.
  *                    This method is called right before the base class will
  *                    start processing. Dynamic properties or other delayed
@@ -283,6 +283,9 @@ gboolean	gst_base_transform_is_qos_enabled   (GstBaseTransform *trans);
 void            gst_base_transform_set_gap_aware    (GstBaseTransform *trans,
                                                      gboolean gap_aware);
 
+void            gst_base_transform_set_prefer_passthrough (GstBaseTransform *trans,
+                                                           gboolean prefer_passthrough);
+
 GstBufferPool * gst_base_transform_get_buffer_pool  (GstBaseTransform *trans);
 void            gst_base_transform_get_allocator    (GstBaseTransform *trans,
                                                      GstAllocator **allocator,
@@ -290,6 +293,8 @@ void            gst_base_transform_get_allocator    (GstBaseTransform *trans,
 
 void		gst_base_transform_reconfigure_sink (GstBaseTransform *trans);
 void		gst_base_transform_reconfigure_src  (GstBaseTransform *trans);
+gboolean gst_base_transform_update_src_caps (GstBaseTransform *trans,
+                                             GstCaps *updated_caps);
 G_END_DECLS
 
 #endif /* __GST_BASE_TRANSFORM_H__ */
